#include "postgresql-libpq-6.2.x.hh"

#line 1 "test-1.uesql.cc"
/*
*  Name:         test-1.uesql.cc
*  Author:       Rafael Jesus Alcantara Perez
*  Contributor:  Francisco Vides Fernandez
*  Summary:      Test for UESQL
*  Date:         $Date: 2001/01/22 20:07:45 $
*  Revision:     $Revision: 1.1 $
*
*  Copyright (C) 1996-1999  Rafael Jesus Alcantara Perez <rafa@dedalo-ing.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <cstdlib>
#include <iostream>
#include <mpcl/config_processor.hh>
#include <mpcl/string.hh>
#include <mpcl/test.h>


//
//  L O C A L   V A R I A B L E S
//

static uesql::TSqlstateDescriptor   _tSqlstateDescriptor;
  

//
//  L O C A L   F U N C T I O N S
//

/**
*  Construct error message from SQL status.
*  @return SQL status message.
*/
static mpcl::xstring _GetErrorMessage (void)
{

  using mpcl::xstring;

  return ( xstring (_tSqlstateDescriptor.getClass())    +
           ": "                                         +
           xstring (_tSqlstateDescriptor.getSubclass()) +
           " (" + uesql::SQLMESSAGE + ")"               );

}  // _GetErrorMessage()


static int _TestCursors (void)
{

  using mpcl::xstring;
  using std::cerr;
  using std::endl;
  using uesql::TSqlInteger;
  using uesql::TSqlString;
  
  TSqlString    yVar0;
  TSqlString    yVar1;
  TSqlString    yVar2;
  TSqlInteger   tZero (0L);
  bool          gEmpty       = true;
  TSqlString    yAttribName  = "WORK_CODE";
  size_t        zRecordCount = 0;

  TEST_INIT ("test for cursors");

#line 82 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 83 "test-1.uesql.cc"

#line 84 "test-1.uesql.cc"

#line 88 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

uesql::htPostgresql62xManager.begin();
ptPgresult = uesql::htPostgresql62xManager.execute ("DECLARE MYCURSOR CURSOR FOR SELECT NAME, HOURS FROM WORKERS WHERE ( NAME = 'nobody' );");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 89 "test-1.uesql.cc"

}  // not PostgreSQL LIBPQ 6.2.x
#line 90 "test-1.uesql.cc"

  while ( true )
  {
#line 94 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 95 "test-1.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar0);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar1);

ptPgresult = uesql::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesql::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesql::htPostgresql62xManager.isNotFoundCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto loop_end;
}
else if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::log << mpcl::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::log << ");\n";
#endif  // not _UESQL_DEBUG__
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 96 "test-1.uesql.cc"

    gEmpty = false;
  }

loop_end:
#line 102 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 103 "test-1.uesql.cc"
uesql::htPostgresql62xManager.clearQueues();

uesql::htPostgresql62xManager.executeClean ("CLOSE MYCURSOR;");
uesql::htPostgresql62xManager.end();

}  // not PostgreSQL LIBPQ 6.2.x
#line 104 "test-1.uesql.cc"

  if ( gEmpty )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }

#line 115 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 119 "test-1.uesql.cc"

#line 120 "test-1.uesql.cc"

#line 125 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (yAttribName);

uesql::htPostgresql62xManager.begin();
ptPgresult = uesql::htPostgresql62xManager.execute ("DECLARE MYCURSOR CURSOR FOR SELECT $z, NAME, HOURS FROM WORKERS;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 126 "test-1.uesql.cc"

}  // not PostgreSQL LIBPQ 6.2.x
#line 127 "test-1.uesql.cc"

    
  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "WORK_CODE NAME                                     HOURS\n");
    fprintf (hptTestTargetFile, "========= ======================================== =====\n");
  }
#line 135 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 136 "test-1.uesql.cc"
{
PGresult*       ptPgresult;
mpcl::xstring   hySqlInstructionAux;
mpcl::xstring   hyCursorName = "MYCURSOR";
int             hI           = 0;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (tZero);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar0);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar1);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar2);

{
PGresult*   ptPgresult;

ptPgresult = uesql::htPostgresql62xManager.execute (mpcl::Format ("MOVE BACKWARD ALL IN %s;", hyCursorName.c_str()));
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
if ( 0 != std::atoi (uesql::htPostgresql62xManager.instantiate ("$z").c_str()) )
{
  ptPgresult = uesql::htPostgresql62xManager.execute (mpcl::Format ("MOVE FORWARD $z IN %s;", hyCursorName.c_str()));
  uesql::htPostgresql62xManager.diagnose (ptPgresult);
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
}
}
ptPgresult = uesql::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesql::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesql::htPostgresql62xManager.isNotFoundCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto loop_end_2;
}
else if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::log << mpcl::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::log << ");\n";
#endif  // not _UESQL_DEBUG__
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 137 "test-1.uesql.cc"

  while ( true )
  {
#line 141 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 142 "test-1.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar0);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar1);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar2);

ptPgresult = uesql::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesql::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesql::htPostgresql62xManager.isNotFoundCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto loop_end_2;
}
else if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::log << mpcl::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::log << ");\n";
#endif  // not _UESQL_DEBUG__
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 143 "test-1.uesql.cc"

    if ( hptTestTargetFile )
    {
      fprintf (hptTestTargetFile, "%9s %40s %5s\n", yVar0.c_str(), yVar1.c_str(), yVar2.c_str());
    }
    ++zRecordCount;
  }

loop_end_2:
#line 153 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 154 "test-1.uesql.cc"

#line 155 "test-1.uesql.cc"
{
PGresult*       ptPgresult;
mpcl::xstring   hySqlInstructionAux;
mpcl::xstring   hyCursorName = "MYCURSOR";
int             hI           = 0;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (tZero);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar0);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar1);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar2);

{
PGresult*   ptPgresult;

ptPgresult = uesql::htPostgresql62xManager.execute (mpcl::Format ("MOVE BACKWARD ALL IN %s;", hyCursorName.c_str()));
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
if ( 0 != std::atoi (uesql::htPostgresql62xManager.instantiate ("$z").c_str()) )
{
  ptPgresult = uesql::htPostgresql62xManager.execute (mpcl::Format ("MOVE FORWARD $z IN %s;", hyCursorName.c_str()));
  uesql::htPostgresql62xManager.diagnose (ptPgresult);
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
}
}
ptPgresult = uesql::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesql::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesql::htPostgresql62xManager.isNotFoundCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto loop_end_3;
}
else if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::log << mpcl::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::log << ");\n";
#endif  // not _UESQL_DEBUG__
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 156 "test-1.uesql.cc"


loop_end_3:
#line 160 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 164 "test-1.uesql.cc"
uesql::htPostgresql62xManager.clearQueues();

uesql::htPostgresql62xManager.executeClean ("CLOSE MYCURSOR;");
uesql::htPostgresql62xManager.end();

}  // not PostgreSQL LIBPQ 6.2.x
#line 165 "test-1.uesql.cc"

  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);  

end:
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;
  
}  // _TestCursors()


static int _TestSqlTypes (void)
{

  TEST_INIT ("test for SQL types");

  using std::cerr;
  using std::endl;
  using std::size_t;
  using uesql::TSqlDate;
  using uesql::TSqlInteger;
  using uesql::TSqlReal;
  using uesql::TSqlString;

  TSqlDate      tSqlDate;
  TSqlInteger   tSqlInteger;
  TSqlReal      tSqlReal;
  TSqlString    ySqlString;
  size_t        zRecordCount = 0;

  TEST_NUMBERS (1,     TSqlInteger ("1"));
  TEST_NUMBERS (0,     TSqlInteger ("0"));
  TEST_NUMBERS (10000, TSqlInteger ("10000"));
  TEST_NUMBERS (10001, TSqlInteger (10001));
  tSqlInteger.getBuffer (12);
  tSqlInteger.getBuffer (12);
  tSqlInteger.getBuffer (12);
  tSqlInteger.set ("75000");
  TEST_NUMBERS (75000, tSqlInteger);
#line 210 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 211 "test-1.uesql.cc"

#line 212 "test-1.uesql.cc"

#line 219 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("CREATE TABLE TABLE_1 (CODE CHAR (3) NOT NULL, BIRTH_DATE DATE, SALARY INT4, PERCENTAGE FLOAT8);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 220 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO TABLE_1 VALUES ('000', '1999-12-31', 12000, 25.75);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 221 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO TABLE_1 VALUES ('001', '1970-01-01', 75000, 30);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 222 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO TABLE_1 VALUES ('002', '2000-01-01', 11500, 99.99);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 226 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (TSqlDate (2000, 12, 31));
uesql::htPostgresql62xManager.pushOnInputQueue (TSqlInteger (7500));
uesql::htPostgresql62xManager.pushOnInputQueue (TSqlReal (33.33));

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO TABLE_1 VALUES ('003', $z, $z, $z);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 230 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (TSqlDate (2001,  1,  1));
uesql::htPostgresql62xManager.pushOnInputQueue (TSqlInteger (6500));
uesql::htPostgresql62xManager.pushOnInputQueue (TSqlReal (0.0));

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO TABLE_1 VALUES ('004', $z, $z, $z);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 232 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

uesql::htPostgresql62xManager.begin();
ptPgresult = uesql::htPostgresql62xManager.execute ("DECLARE MYCURSOR CURSOR FOR SELECT CODE, BIRTH_DATE, SALARY, PERCENTAGE FROM TABLE_1;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 233 "test-1.uesql.cc"

}  // not PostgreSQL LIBPQ 6.2.x
#line 234 "test-1.uesql.cc"


  //
  //  Writes the example table.
  //
  if ( hptTestTargetFile )
  {
    fprintf (hptTestTargetFile, "%s\n", "CODE BIRTH_DATE   SALARY PERCENTAGE");
    fprintf (hptTestTargetFile, "%s\n", "==== ============ ====== ==========");
  }
  while ( true )
  {
#line 247 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 248 "test-1.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnOutputQueue (&ySqlString);
uesql::htPostgresql62xManager.pushOnOutputQueue (&tSqlDate);
uesql::htPostgresql62xManager.pushOnOutputQueue (&tSqlInteger);
uesql::htPostgresql62xManager.pushOnOutputQueue (&tSqlReal);

ptPgresult = uesql::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesql::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesql::htPostgresql62xManager.isNotFoundCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto loop_end;
}
else if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::log << mpcl::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::log << ");\n";
#endif  // not _UESQL_DEBUG__
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 249 "test-1.uesql.cc"

    if ( hptTestTargetFile )
    {
      fprintf ( hptTestTargetFile          ,
                "%3s  %12s %6ld %10.03f\n" ,
                ySqlString.c_str()         ,
                tSqlDate.get().c_str()     ,
                (long int) tSqlInteger     ,
                (double) tSqlReal          );
    }
    ++zRecordCount;
  }

loop_end:
#line 264 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 265 "test-1.uesql.cc"
uesql::htPostgresql62xManager.clearQueues();

uesql::htPostgresql62xManager.executeClean ("CLOSE MYCURSOR;");
uesql::htPostgresql62xManager.end();

#line 266 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("DROP TABLE TABLE_1;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 267 "test-1.uesql.cc"

  if ( zRecordCount == 5 )
  {
    TEST_PASS;
  }
  else
  {
    TEST_FAIL;
  }
  goto end;

error:
  cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
  cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
  cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
  TEST_FAIL_WITH_CODE (uesql::SQLCODE);  

end:  
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestSqlTypes()


/**
*  Main.
*
*  Parameters:
*
*    {-o, --dbms-server-name}=DBMS-SERVER-NAME
*    {-u, --dbms-user-name}=DBMS-USER-NAME
*    {-p, --dbms-user-password}=PASSWORD
*/
int main (int argc, const char* argv[])
{

  using mpcl::Format;
  using mpcl::TConfigProcessor;
  using mpcl::xstring;
  using std::cerr;
  using std::endl;
  using uesql::ComposeQuotedSchemaLocation;
  using uesql::ComposeSchemaLocation;
  using uesql::ComposeUserLogin;
  using uesql::Quote;
  using uesql::TSqlInteger;
  using uesql::TSqlString;
  
  TEST_INIT ("test-1");
  TEST_START_WATCH;

  TSqlInteger        tInteger;
  TSqlString         yVar0;
  TSqlString         yVar1;
  TSqlString         yVar2;
  TSqlString         yCursorName;
  TSqlString         yDbmsUserLogin;
  TSqlString         yDbmsSchemaLocation;
  TSqlString         yDbmsServerName;
  TConfigProcessor   tConfigProcessor (argc, argv);
  TSqlString         yCodeValue ("002");
  const char*        pcDbmsServerName = getenv ("DBMS_SERVER_NAME");
  const char*        pcDbmsUserName   = getenv ("DBMS_USER_NAME");
  TSqlString*        pyDbmsUserLogin  = &yDbmsUserLogin;

  try
  {
    //
    //  Checks environment variables.
    //
    if ( !pcDbmsServerName )
    {
      pcDbmsServerName = "localhost";
    }
    if ( !pcDbmsUserName )
    {
      pcDbmsUserName = getenv ("USER");
      if ( !pcDbmsUserName )
      {
        pcDbmsUserName = "";
      }
    }
    
    //
    //  Define string constants names for options.
    //
    tConfigProcessor.addOption ("dbms-server-name",   "o", pcDbmsServerName);
    tConfigProcessor.addOption ("dbms-user-name",     "u", pcDbmsUserName);
    tConfigProcessor.addOption ("dbms-user-password", "p", pcDbmsUserName);
    tConfigProcessor.processOptions();
    
    yDbmsServerName     = tConfigProcessor ["dbms-server-name"].value();
    yDbmsUserLogin      = tConfigProcessor ["dbms-user-name"].value();
    yDbmsSchemaLocation = ComposeSchemaLocation
                          (
                            yDbmsServerName                             ,
                            tConfigProcessor ["dbms-user-name"].value() 
                          );

    //
    //  If there is a password then appends it to the user-login string.
    //
    if ( !tConfigProcessor ["dbms-user-password"].value().empty() )
    {
      yDbmsUserLogin = ComposeUserLogin
                       (
                         yDbmsUserLogin                                  ,
                         tConfigProcessor ["dbms-user-password"].value() 
                       );
    }
  }
  catch (const mpcl::TBaseException& rktEXCEPTION)
  {
    cerr << rktEXCEPTION;
    TEST_FAIL_WITH_CODE (3);
  }

  TEST_WRITE_INFO ("DBMS-SCHEMA-LOCATION = %s", yDbmsSchemaLocation.c_str());
  TEST_WRITE_INFO ("DBMS-USER-LOGIN      = %s", yDbmsUserLogin.c_str());
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::log.also (cerr);
  }
  if ( !yDbmsServerName.empty() && !yDbmsUserLogin.empty() )
  {
#line 393 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 397 "test-1.uesql.cc"

#line 398 "test-1.uesql.cc"

#line 404 "test-1.uesql.cc"
{
mpcl::xstring   hySqlServerDescriptor;
mpcl::xstring   hySqlServerName;
mpcl::xstring   hyDatabaseName;
mpcl::xstring   hyConnectionName;
mpcl::xstring   hyUserLogin;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (Quote (yDbmsSchemaLocation));
uesql::htPostgresql62xManager.pushOnInputQueue (Quote (yDbmsUserLogin));

hySqlServerDescriptor = uesql::htPostgresql62xManager.instantiate ("$z");
hyConnectionName      = "DEFAULT";
hyUserLogin           = uesql::htPostgresql62xManager.instantiate ("$z");
hySqlServerDescriptor.suppress ('\'');
if ( hySqlServerDescriptor.find ('/') == mpcl::xstring::npos )
{
  hySqlServerName = hySqlServerDescriptor;
}
else
{
  hySqlServerName = hySqlServerDescriptor.substr (0, hySqlServerDescriptor.find ('/'));
  hyDatabaseName  = hySqlServerDescriptor.substr (hySqlServerDescriptor.find ('/') + 1);
}
if ( uesql::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  uesql::htPostgresql62xManager.disconnect (hyConnectionName);
}
if ( uesql::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08002";
  goto error;
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesql::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesql::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 413 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("CREATE TABLE JOBS (CODE CHAR (3) NOT NULL, WORKER_COUNT INT4);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 420 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("CREATE TABLE WORKERS (WORK_CODE CHAR (3) NOT NULL, NAME CHAR (40) NOT NULL, HOURS DECIMAL (5));");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 425 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("ALTER TABLE WORKERS ADD COLUMN STATE CHAR (40);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 426 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("ALTER TABLE WORKERS ADD COLUMN PHONE_NUMBER INT4;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 431 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (Quote (yCodeValue));

ptPgresult = uesql::htPostgresql62xManager.execute ("DELETE FROM JOBS WHERE ( CODE = $z );");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 432 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("DELETE FROM JOBS WHERE ( CODE = '001' );");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 433 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("DELETE FROM WORKERS;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 438 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO JOBS VALUES ('001', 100);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 439 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO JOBS VALUES ('002', 250);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 440 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'myname', 40, 'Andalucia', 5550000);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 441 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'yourname', 35, 'Extremadura', 5550001);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 442 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('001', 'hisname', 25, 'Catalunya', 5550002);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 443 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'john', 35, 'New England', 5550003);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 444 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'james', 35, 'Washington', 5550004);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 445 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'william', NULL, 'Missouri', 5550005);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 446 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO WORKERS VALUES ('002', 'peter', NULL, 'Arkansas', 5550006);");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 447 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS WHERE ( NAME = 'myname' );");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 448 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("INSERT INTO WORKERS SELECT WORK_CODE, NAME, HOURS, STATE, PHONE_NUMBER FROM WORKERS;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 453 "test-1.uesql.cc"

#line 460 "test-1.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar1);

uesql::htPostgresql62xManager.begin();
ptPgresult = uesql::htPostgresql62xManager.execute ("DECLARE AUX_CURSOR CURSOR FOR SELECT HOURS FROM WORKERS, JOBS WHERE ( WORK_CODE = CODE );");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( !uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  ptPgresult = uesql::htPostgresql62xManager.execute ("FETCH ALL IN AUX_CURSOR;");
  uesql::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
}
if ( uesql::htPostgresql62xManager.isNotFoundCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  uesql::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesql::htPostgresql62xManager.end();
  goto not_found;
}
else if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  uesql::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesql::htPostgresql62xManager.end();
  goto cardinality_check;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::log << mpcl::header << "SELECTED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::log << ");\n";
#endif  // not _UESQL_DEBUG__
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  uesql::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesql::htPostgresql62xManager.end();
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 461 "test-1.uesql.cc"

    TEST_FAIL;

cardinality_check:
    TEST_STRINGS (uesql::SQLSTATE.c_str(), "21000");
#line 467 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 471 "test-1.uesql.cc"

#line 476 "test-1.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnOutputQueue (&tInteger);

uesql::htPostgresql62xManager.begin();
ptPgresult = uesql::htPostgresql62xManager.execute ("DECLARE AUX_CURSOR CURSOR FOR SELECT COUNT (*) FROM WORKERS WHERE ( HOURS IS NULL );");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( !uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  ptPgresult = uesql::htPostgresql62xManager.execute ("FETCH ALL IN AUX_CURSOR;");
  uesql::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
}
if ( uesql::htPostgresql62xManager.isNotFoundCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  uesql::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesql::htPostgresql62xManager.end();
  goto not_found;
}
else if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  uesql::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesql::htPostgresql62xManager.end();
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::log << mpcl::header << "SELECTED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::log << ");\n";
#endif  // not _UESQL_DEBUG__
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  uesql::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesql::htPostgresql62xManager.end();
}
}
#line 481 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

uesql::htPostgresql62xManager.begin();
ptPgresult = uesql::htPostgresql62xManager.execute ("DECLARE MYCURSOR CURSOR FOR SELECT WORK_CODE, HOURS FROM WORKERS WHERE ( HOURS IS NULL );");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 482 "test-1.uesql.cc"

#line 483 "test-1.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar0);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar1);

ptPgresult = uesql::htPostgresql62xManager.execute ("FETCH IN MYCURSOR;");
uesql::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesql::htPostgresql62xManager.isNotFoundCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto not_found;
}
else if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::log << mpcl::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
  if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << ", " << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::log << ");\n";
#endif  // not _UESQL_DEBUG__
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
}
}
#line 484 "test-1.uesql.cc"
uesql::htPostgresql62xManager.clearQueues();

uesql::htPostgresql62xManager.executeClean ("CLOSE MYCURSOR;");
uesql::htPostgresql62xManager.end();

}  // not PostgreSQL LIBPQ 6.2.x
#line 485 "test-1.uesql.cc"

    TEST_NUMBERS (tInteger,          4);
    TEST_NUMBERS (tInteger.isNull(), false);
    TEST_NUMBERS (yVar0.isNull(),    false);
    TEST_NUMBERS (yVar1.isNull(),    true);
    
    yCursorName = "MYCURSOR";
#line 493 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 497 "test-1.uesql.cc"

#line 502 "test-1.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar2);

uesql::htPostgresql62xManager.begin();
ptPgresult = uesql::htPostgresql62xManager.execute ("DECLARE AUX_CURSOR CURSOR FOR SELECT COUNT (*) FROM WORKERS WHERE ( NAME = 'myname' );");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( !uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  ptPgresult = uesql::htPostgresql62xManager.execute ("FETCH ALL IN AUX_CURSOR;");
  uesql::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
}
if ( uesql::htPostgresql62xManager.isNotFoundCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  uesql::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesql::htPostgresql62xManager.end();
  goto not_found;
}
else if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  uesql::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesql::htPostgresql62xManager.end();
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::log << mpcl::header << "SELECTED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::log << ");\n";
#endif  // not _UESQL_DEBUG__
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  uesql::htPostgresql62xManager.executeClean ("CLOSE AUX_CURSOR;");
  uesql::htPostgresql62xManager.end();
}
}
#line 509 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("UPDATE WORKERS SET PHONE_NUMBER = WORKERS.PHONE_NUMBER WHERE ( ( WORKERS.NAME = 'myname' ) AND ( WORKERS.NAME = NAME ) );");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 510 "test-1.uesql.cc"

    TEST_FUNCTION (_TestCursors());
    TEST_FUNCTION (_TestSqlTypes());
#line 514 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 518 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

uesql::htPostgresql62xManager.begin();
ptPgresult = uesql::htPostgresql62xManager.execute ("DECLARE MYCURSOR CURSOR FOR SELECT CODE FROM JOBS WHERE ( CODE = '001' );");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 519 "test-1.uesql.cc"

#line 520 "test-1.uesql.cc"
{
PGresult*   ptPgresult;
int         hI = 0;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (yCursorName);
uesql::htPostgresql62xManager.pushOnOutputQueue (&yVar0);

ptPgresult = uesql::htPostgresql62xManager.execute ("FETCH IN $z;");
uesql::htPostgresql62xManager.diagnoseSingleRow (ptPgresult);
if ( uesql::htPostgresql62xManager.isNotFoundCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto not_found;
}
else if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
  goto error;
}
else
{
#ifdef _UESQL_DEBUG__
  mpcl::log << mpcl::header << "FETCHED (";
#endif  // not _UESQL_DEBUG__
if ( PQgetisnull (ptPgresult, 0, hI) )
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (NULL);
  }
  else
  {
    uesql::htPostgresql62xManager.getFromOutputQueue (hI)->set (PQgetvalue (ptPgresult, 0, hI));
  }
# ifdef _UESQL_DEBUG__
  mpcl::log << uesql::htPostgresql62xManager.getFromOutputQueue (hI)->get();
# endif  // not _UESQL_DEBUG__
  ++hI;
#ifdef _UESQL_DEBUG__
  mpcl::log << ");\n";
#endif  // not _UESQL_DEBUG__
  uesql::htPostgresql62xManager.clearResult (ptPgresult);
}
}
#line 521 "test-1.uesql.cc"
uesql::htPostgresql62xManager.clearQueues();

uesql::htPostgresql62xManager.executeClean ("CLOSE MYCURSOR;");
uesql::htPostgresql62xManager.end();

}  // not PostgreSQL LIBPQ 6.2.x
#line 522 "test-1.uesql.cc"

    TEST_STRINGS (uesql::SQLSTATE.c_str(), "00000");
    TEST_STRINGS (yVar2.c_str(),           "4");
    TEST_STRINGS (yVar0.c_str(),           "001");
#line 527 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 531 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("REVOKE ALL ON JOBS FROM PUBLIC;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 532 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'RESTRICT' has been omitted.
//
uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("REVOKE ALL ON WORKERS FROM PUBLIC;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 537 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("GRANT SELECT ON JOBS TO PUBLIC;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 538 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("GRANT SELECT ON WORKERS TO PUBLIC;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 539 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("GRANT DELETE ON WORKERS TO PUBLIC;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 544 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("CREATE VIEW VIEW1 AS SELECT * FROM JOBS;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 549 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("DROP VIEW VIEW1;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 554 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("DROP TABLE JOBS;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
#line 559 "test-1.uesql.cc"
{
uesql::htPostgresql62xManager.clearQueues();

if ( uesql::htPostgresql62xManager.existsDefaultConnection() )
{
  uesql::htPostgresql62xManager.resetStatus();
  uesql::htPostgresql62xManager.disconnectCurrent();
}
else
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection does not exist
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08003";
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 560 "test-1.uesql.cc"


#line 563 "test-1.uesql.cc"
{
//
//  PostgreSQL LIBPQ 6.2.x instructions.
//
#line 569 "test-1.uesql.cc"
{
mpcl::xstring   hySqlServerDescriptor;
mpcl::xstring   hySqlServerName;
mpcl::xstring   hyDatabaseName;
mpcl::xstring   hyConnectionName;
mpcl::xstring   hyUserLogin;

uesql::htPostgresql62xManager.clearQueues();
uesql::htPostgresql62xManager.pushOnInputQueue (Quote (yDbmsSchemaLocation));
uesql::htPostgresql62xManager.pushOnInputQueue (Quote (*pyDbmsUserLogin));

hySqlServerDescriptor = uesql::htPostgresql62xManager.instantiate ("$z");
hyConnectionName      = uesql::htPostgresql62xManager.instantiate ("_'myconnection'");
hyUserLogin           = uesql::htPostgresql62xManager.instantiate ("$z");
hySqlServerDescriptor.suppress ('\'');
hyConnectionName.suppress ('\'');
if ( hySqlServerDescriptor.find ('/') == mpcl::xstring::npos )
{
  hySqlServerName = hySqlServerDescriptor;
}
else
{
  hySqlServerName = hySqlServerDescriptor.substr (0, hySqlServerDescriptor.find ('/'));
  hyDatabaseName  = hySqlServerDescriptor.substr (hySqlServerDescriptor.find ('/') + 1);
}
if ( uesql::htPostgresql62xManager.isConnected (hyConnectionName) )
{
  //
  //  Condition:           connection exception
  //  Class subcondition:  connection name in use
  //
  uesql::SQLCODE    = -1;
  uesql::SQLMESSAGE = "";
  uesql::SQLSTATE   = "08002";
  goto error;
}
else
{
  if ( hyDatabaseName.empty() )
  {
    //
    //  If there is no PostgreSQL database then connect to 'template1'
    //  (that should always exists).
    //
    hyDatabaseName = "template1";
  }
  uesql::htPostgresql62xManager.connect ( hyConnectionName ,
                                     hySqlServerName  ,
                                     hyDatabaseName   ,
                                     hyUserLogin      );
  if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
  {
    goto error;
  }
  else
  {
    uesql::htPostgresql62xManager.setCurrentConnection (hyConnectionName);
  }
}
}
#line 574 "test-1.uesql.cc"
{
PGresult*   ptPgresult;

//
//  'CASCADE' has been omitted.
//
uesql::htPostgresql62xManager.clearQueues();

ptPgresult = uesql::htPostgresql62xManager.execute ("DROP TABLE WORKERS;");
uesql::htPostgresql62xManager.diagnose (ptPgresult);
uesql::htPostgresql62xManager.clearResult (ptPgresult);
if ( uesql::htPostgresql62xManager.isSqlerrorCondition() )
{
  goto error;
}
}
}  // not PostgreSQL LIBPQ 6.2.x
#line 575 "test-1.uesql.cc"

    goto end;

  error:
    cerr << "SQLSTATE="    << uesql::SQLSTATE    << endl;
    cerr << "SQLCODE="     << uesql::SQLCODE     << endl;
    cerr << "DESCRIPTION=" << _GetErrorMessage() << endl;
    TEST_FAIL_WITH_CODE (uesql::SQLCODE);
    goto end;

  not_found:
    cerr << "Search failed\n";
    TEST_FAIL_WITH_CODE (2);
    goto end;
  }

end:
  if ( !TEST_IN_SILENT_MODE )
  {
    mpcl::log.erase (cerr);
  }
  TEST_STOP_WATCH;
  TEST_DISPLAY_WATCH ("UESQL benchmark");
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
